{ Demo program for ChroMathsome - Linear Algebra.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra.
  As an example, we will rotate points (vectors) in a two-dimensional plane and
  resize a bitmap picture.

  Note: when running this demo, maximize the window if your screen resolution is lower
        than 1024*768 or you may miss parts of the "wizzardry".

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "R".              }
unit rotmat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LinAlg, Menus, ExtCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    Draw1: TMenuItem;
    Go1: TMenuItem;
    Image1: TImage;
    procedure Close1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Go1Click(Sender: TObject);
  private
    { Private declarations }
    procedure PauseEvent(Sec: integer);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  Vec: RealVector;

implementation

{$R *.DFM}


procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     InitRealVector(Vec, 2);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     DestroyRealVector(Vec);
end;

procedure TForm1.PauseEvent(Sec: integer);
var t1, j: longint;
begin
     t1:= gettickcount;
     j:= 1;
     repeat
     begin
     j:= j;
      application.ProcessMessages;
     end;
     until (gettickcount - t1) > Sec;
end;

{ Rotate the picture in Image1 over an angle of pi/4 3*pi/4 and 5*pi/4 radians.
  Resize the bitmap.
  Note that we must use integers to draw on a canvas (note the Round(...) function below).
  This, of course, results in a decrease of accuracy of the drawed picture.
  Also, the color depth is not 100% correct (limitation of TCanvas object?). }
procedure TForm1.Go1Click(Sender: TObject);
var i, j: integer;
    M1, M2, RotMat: RealMatrix;
    CM1: ComplexMatrix;
    Angle: extended;
    LColor: TColor;
begin
     InitRealMatrix(RotMat, 2, 2);
     InitRealMatrix(M1, 1, 2);
     InitRealMatrix(M2, 2, 2);
     InitComplexMatrix(CM1, 1, 1);
     try
     Angle:= pi/4;
     TwoDimensionalRotationRealMatrix(RotMat, Angle);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     begin
     LColor:= Image1.Canvas.Pixels[i, j];
     Vec.Vector^[1]:= i;
     Vec.Vector^[2]:= j;
     RealVectorToRealMatrix(M1, Vec);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(M2, RotMat, M1);
     RealMatrixToRealVector(Vec, M2);
     Form1.Canvas.Pixels[500 + Round(Vec.Vector^[1]), 250 + Round(Vec.Vector^[2])]:= LColor;
     end;

     Form1.Caption:= 'Picture rotated over angle pi/4 radians.';
     PauseEvent(3000);
     Form1.Refresh;

     Angle:= 3*pi/4;
     TwoDimensionalRotationRealMatrix(RotMat, Angle);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     begin
     LColor:= Image1.Canvas.Pixels[i, j];
     Vec.Vector^[1]:= i;
     Vec.Vector^[2]:= j;
     RealVectorToRealMatrix(M1, Vec);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(M2, RotMat, M1);
     RealMatrixToRealVector(Vec, M2);
     Form1.Canvas.Pixels[500+ Round(Vec.Vector^[1]), 450 + Round(Vec.Vector^[2])]:= LColor;
     end;

     Form1.Caption:= 'Picture rotated over angle 3*pi/4 radians.';
     PauseEvent(3000);
     Form1.Refresh;

     Angle:= 5*pi/4;
     TwoDimensionalRotationRealMatrix(RotMat, Angle);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     begin
     LColor:= Image1.Canvas.Pixels[i, j];
     Vec.Vector^[1]:= i;
     Vec.Vector^[2]:= j;
     RealVectorToRealMatrix(M1, Vec);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(M2, RotMat, M1);
     RealMatrixToRealVector(Vec, M2);
     Form1.Canvas.Pixels[500 + Round(Vec.Vector^[1]), 450 + Round(Vec.Vector^[2])]:= LColor;
     end;

     Form1.Caption:= 'Picture rotated over angle 5*pi/4 radians.';
     PauseEvent(3000);
     Form1.Refresh;

    { Draw the bitmap 3/4 the size of the original. }
     Redimension_ComplexMatrix(CM1, Image1.Width, Image1.Height);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     begin
     CM1.Matrix^[i]^[j].re:= i;
     CM1.Matrix^[i]^[j].im:= j;
     end;
     ComplexMatrix_RealScalarMultiplication(CM1, 0.75);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     begin
     LColor:= Image1.Canvas.Pixels[i, j];
     Form1.Canvas.Pixels[250+ Round(CM1.Matrix^[i]^[j].re), Round(CM1.Matrix^[i]^[j].im)]:= LColor;
     end;

     Form1.Caption:= 'ChroMathsome - Demo';

     finally
     DestroyRealMatrix(RotMat);
     DestroyRealMatrix(M1);
     DestroyRealMatrix(M2);
     DestroyComplexMatrix(CM1);
     end;
end;

end.
